/*
 * Copyright 2017 Collective Financial Services Inc.
 */
package web.base.data

import scala.scalajs.js.UndefOr
import scala.scalajs.js.JSConverters._

case class FieldError(status: String, message: String)

case class Field[A](initialValue: Option[Either[FieldError, A]]) {
  def getOrElse[B >: A](default: => B): B = {
    val res: Option[B] = initialValue.flatMap[B](_.right.toOption)
    res.getOrElse[B](default)
  }

  def get: Option[A] = {
    initialValue.flatMap(_.right.toOption)
  }

  def getUndef: UndefOr[A] = {
    initialValue.flatMap(_.right.toOption).orUndefined
  }

  def getError: Option[FieldError] = {
    initialValue.flatMap(_.left.toOption)
  }

  def nonEmpty: Boolean = {
    initialValue.flatMap(_.right.toOption).nonEmpty
  }

//  def validateStatus: Option[String] =
//    initialValue.flatMap(e => e.right.toOption.map(_ => "success").orElse(e.left.toOption.map(_ => "error")))
}

object Field {
  def empty[A]: Field[A] = new Field[A](None)

  def apply[A](value: A): Field[A]          = new Field[A](Option(Right(value)))
  def apply[A](value: FieldError): Field[A] = new Field[A](Option(Left(value)))
}
