package web.cps

import ant.cps.Form._
import ant.cps.Form.FormComponent.ValidatorFn
import ant.cps.{Form, Input}
import google.maps.{LatLngBounds, MapsEventListener}
import google.maps.places.{PlaceResult, SearchBox}
import japgolly.scalajs.react._
import japgolly.scalajs.react.component.Js
import japgolly.scalajs.react.vdom.html_<^._
import monix.eval.{MVar, Task}
import monix.execution.CancelableFuture
import org.scalajs.dom
import org.scalajs.dom.html
import shared.models.Flds
import shared.models.a1.FieldKey
import web.base.all._
import web.base.data.PlaceFrag

import scala.scalajs.js
import scala.scalajs.js.annotation.ScalaJSDefined
import scala.scalajs.js.|
import scala.scalajs.js.JSConverters._

object AddressInputSuggest {

  sealed trait TouchState
  object TouchState {
    case object Untouched extends TouchState
    case object Touched extends TouchState
    case object Submitted extends TouchState
  }

  @js.native
  trait InputRef extends js.Object { def input: html.Input }

  @js.native
  // TODO: Change this absolutely horrid name.
  trait RefHouse[T <: js.Object] extends js.Object { def refs: T }

  case class Props(changed: Seq[PlaceResult] => Callback,
                   searchMounted: Option[SearchBox] => Callback,
                   defaultPlace: Option[PlaceFrag],
                   form: FormProps,
                   submitForm: Option[Task[Unit]] = None)
      extends FormFacade
  case class State(place: Option[PlaceResult])

  class Backend(val $ : BackendScope[Props, State]) {
    import TouchState._

    var search: SearchBox           = _
    var listener: MapsEventListener = _
    var inputTouched: TouchState    = Untouched
    var inputCb                     = List.empty[Task[Unit]]

    /* Specifically for places and the interface with google autocomplete */
    // TODO: Potentially addFacade of RefHouse to Input element on Ant.
    private val inputRef = Js.mutableRefTo(Input.C.addFacade[RefHouse[InputRef]])

    // ====================== Event Handlers =====================
    def mounted() = $.props.flatMap { p =>
      search   = new SearchBox(inputRef.value.raw.refs.input)
      listener = search.addListener("places_changed", placesChanged.toJsFn)
      p.searchMounted(Some(search))
    }

    def unmount() = $.props.flatMap { p =>
      google.maps.event.removeListener(listener)
      p.searchMounted(None)
    }

    /**
      * The first level of placesChanged, it deals with places and validating the data.
      */
    val placesChanged = $.props.flatMap { p =>
      println("placesChanged")

      search.getPlaces() match {
        case places: js.Array[PlaceResult @unchecked] =>
          // Call even if empty, but not on undef'd
          val address = places.head.formatted_address
          p.form.setFields(js.Dictionary(FieldOptions.tupledTouched(FieldKey.Place, address)))

          // Run all the waiting callbacks, then assign it all nil

          //
          p.changed(places) >>
            $.modState(_.copy(place = places.headOption)) >>
            Callback {
              Task.sequence(inputCb).runAsync
              inputCb = Nil
            }

        case x if js.isUndefined(x) =>
          Callback.log("Not places found")
      }
    }

    /**
      * Handle overriding the return field and
      * @param r
      * @return
      */
    def onChange(r: ReactKeyboardEventFromInput): Callback = $.props.flatMap { p =>
      r.charCode match {
        // There is no change and there was a resubmit
        case 13 if inputTouched == Untouched => Callback.empty
        // The value is empty, therefore keep going.
        case 13 if r.target.value.isEmpty  => Callback.empty
        case 13 if inputTouched == Touched =>
          // this will always be faster than the result from Google, so attach a waiting callback
          // upon return a place, execute all the callbacks.
          // Set [[Untouched]] to prevent additional callbacks.
          inputTouched = Untouched
          p.submitForm.foreach { cb =>
            inputCb ::= cb.map(_ => println(s"Running, $cb"))
          }
          r.preventDefaultCB.logAround("input touched.")
        case x =>
          println(x)
          inputTouched = Touched
          Callback.empty
      }
    }

    /**
      * There was a hack here at one point to simulate pressing the placesChanged button, but it
      * doesn't seem necessary now because of extra complexities in the and
      * [[placesChanged]] function */
    private def inputOpts(defaultInput: js.UndefOr[String]) =
      new Input.Opt {
        override val placeholder  = "Street Address"
        override val size         = "large"
        override val defaultValue = defaultInput
      }.ex(^.onKeyPress ==> onChange)

    def render(p: Props) = {
      p.form.getFieldDecorator(Flds.place, new FormComponent.Opt {
        override val validateTrigger = null // Only trigger on submit
        override val rules =
          js.defined(js.Array(FormComponent.RuleOpt(required = true, message = "Please enter an address")))
        override val initialValue = p.defaultPlace.map(_.formattedAddress).orUndefined
      })

      val inputHelp   = p.form.getFieldError(Flds.place).asInstanceOf[js.UndefOr[String]]
      val inputStatus = p.form.getFieldError(Flds.place).fold("success")(_ => "error")

      Item(new Item.Opt {
        override val label    = js.defined("Enter your Address")
        override val required = true

        // TODO: Add error and validate status messages here.
        override val help           = inputHelp
        override val validateStatus = inputStatus

      })(inputRef.component(inputOpts(p.defaultPlace.map(_.formattedAddress).orUndefined)))
    }
  }

  val C = ScalaComponent
    .builder[Props]("AddressInputSuggest")
    .initialStateCallbackFromProps(p => CallbackTo(State(None)))
    .renderBackend[Backend]
    .componentDidMount(_.backend.mounted())
    .componentWillUnmount(_.backend.unmount())
    .build
}
