package web.cps.a1quote

import ant.cps.Col
import japgolly.scalajs.react._
import japgolly.scalajs.react.component.builder.Lifecycle
import vdom.html_<^._

import scala.scalajs.js

object FormCtl {
  //
  case class RowP(t: TagMod*)

  val Row = ScalaComponent
    .builder[RowP]("FormCtlRow")
    .stateless
    .render_PC { (p, pc) =>
      <.div(Seq(^.className := "tt-top") ++ p.t: _*)(pc)
    }
    .build

  def row(t: TagMod*)(vn: VdomNode*): Row.Unmounted = Row(RowP(t: _*))(vn: _*)

  // ==================== Tooltip oriented row ====================
  case class TTRowP(toolTip: VdomNode, child: ShowCtl => VdomNode, mods: Seq[TagMod] = Nil)

  /**
    * This is a mutable baton to use to set showing and non showing of the tooltip itself.
    * it allows someone to reach into a component and set the sibling (tooltip) component without
    * having to have a traditional React relationship.
    *
    */
  class ShowCtl {

    private var lc: Lifecycle.ComponentDidMount[TTRowP, ShowCtl, Unit] = _
    private var show: Boolean                                          = false
    private var unInit: Boolean                                        = true

    def setShow(b: Boolean): Callback =
      CallbackTo {
        val oldShow = show
        unInit = false
        show   = b
        oldShow != b
      } >>= {
        case true  => lc.forceUpdate
        case false => Callback.empty
      }

    def getShow: CallbackTo[Boolean]   = CallbackTo(show)
    def genUnInit: CallbackTo[Boolean] = CallbackTo(unInit)

    private[FormCtl] def setLC(lcOpt: Lifecycle.ComponentDidMount[TTRowP, ShowCtl, Unit]): Callback = {
      CallbackTo {
        lc = lcOpt
      }
    }

    /** Helper on component to [[japgolly.scalajs.react.vdom.TagMod]] */
    def toInputTagMod: Seq[TagMod] = Seq(^.onFocus --> setShow(true), ^.onBlur --> setShow(false))
    def toMouseTagMod: Seq[TagMod] = Seq(^.onMouseEnter --> setShow(true), ^.onMouseLeave --> setShow(false))
  }

  //
  val TTRow = ScalaComponent
    .builder[TTRowP]("FormCtlTTRow")
    .initialState(new ShowCtl())
    .render_PS { (p, showCtl) =>
      val divProps = Seq(^.className := "tt") ++ (
        if (showCtl.genUnInit.runNow()) {
          Seq(^.visibility.hidden)
        } else if (showCtl.getShow.runNow()) {
          Seq(^.className := "tt-enter")
        } else {
          Seq(^.className := "tt-leave")
        }
      )

      Row(RowP(p.mods: _*))(<.div(divProps: _*)(p.toolTip), p.child(showCtl))
    }
    .componentDidMount(f => f.state.setLC(f))
    .build

  def ttrow(mods: Seq[TagMod] = Nil)(toolTip: VdomNode)(child: ShowCtl => VdomNode): TTRow.Unmounted = {
    TTRow(TTRowP(toolTip, child, mods))
  }

  case class SplitTTRowP(tm: Seq[TagMod],
                         leftTt: VdomElement,
                         left: ShowCtl => VdomElement,
                         rightTt: VdomElement,
                         right: ShowCtl => VdomElement)

  val SplitTTRow = ScalaComponent
    .builder[SplitTTRowP]("SplitTTRow")
    .stateless
    .render_P { p =>
      //
      ant.cps.Row(gutter = 16)(
        Col(new Col.Opt {
          override val span = 12
          override val sm   = 12
          override val xs   = 24
        })(ttrow(p.tm :+ (^.className := "tt-left"))(p.leftTt)(p.left)),
        Col(new Col.Opt {
          override val span = 12
          override val sm   = 12
          override val xs   = 24
        })(ttrow(p.tm)(p.rightTt)(p.right)),
      )
    }
    .build

  def splitTTRow(mods: TagMod*)(leftTt: VdomElement, rightTt: VdomElement)(
    left: ShowCtl => VdomElement,
    right: ShowCtl => VdomElement): SplitTTRow.Unmounted = {
    //
    SplitTTRow(SplitTTRowP(mods, leftTt, left, rightTt, right))
  }

  case class SplitRowP(mods: Seq[TagMod])

  val SplitRow = ScalaComponent
    .builder[SplitRowP]("SplitRow")
    .stateless
    .render_PC { (p, pc) =>
      var i = 0
      //
      <.div(pc.toList.sliding(2, 2).toVdomArray { ch =>
        i = i + 1
        if (ch.size == 2)
          ant.cps.Row(gutter = 16, key = (i + 1).toString)(Col(new Col.Opt {
            override val span = 12
            override val sm   = 12
            override val xs   = 24
          })(row(p.mods: _*)(ch.head)), Col(new Col.Opt {
            override val span = 12
            override val sm   = 12
            override val xs   = 24
          })(row(p.mods: _*)(ch.tail.head)))
        else if (ch.size == 1)
          ant.cps.Row(gutter = 16, key = (i + 1).toString)(Col(new Col.Opt {})(row(p.mods: _*)(ch.head)))
        else VdomNode.cast(null)
      })

    }
    .build

  def splitRow(mods: TagMod*)(child: VdomNode*): SplitRow.Unmounted = {
    SplitRow(SplitRowP(mods))(child: _*)
  }

  case class ArbSplitTTRowP(tm: Seq[TagMod], children: Seq[(VdomElement, ShowCtl => VdomElement)])

  val ArbSplitTTRow = ScalaComponent
    .builder[ArbSplitTTRowP]("ArbSplitTTRow")
    .stateless
    .render_P { p =>
      var i = 0
      //
      <.div(p.children.toList.sliding(2, 2).toVdomArray { ch =>
        i = i + 1
        if (ch.size == 2)
          ant.cps.Row(gutter = 16, key = (i + 1).toString)(Col(new Col.Opt {
            override val span = 12
            override val sm   = 12
            override val xs   = 24
          })(ttrow(p.tm :+ (^.className := "tt-left"))(ch.head._1)(ch.head._2)), Col(new Col.Opt {
            override val span = 12
            override val sm   = 12
            override val xs   = 24
          })(ttrow(p.tm)(ch.tail.head._1)(ch.tail.head._2)))
        else if (ch.size == 1)
          ant.cps.Row(gutter = 16, key = (i + 1).toString)(Col(new Col.Opt {})(ttrow(p.tm)(ch.head._1)(ch.head._2)))
        else VdomNode.cast(null)
      })

    }
    .build

  //
  def arbSplitTTRow(mods: TagMod*)(child: (VdomElement, ShowCtl => VdomElement)*): ArbSplitTTRow.Unmounted = {
    ArbSplitTTRow(ArbSplitTTRowP(mods, child))
  }

  val AccentRow = ScalaComponent
    .builder[Unit]("AccentRow")
    .stateless
    .render_C { pc =>
      <.div(^.className := "accent-row")(<.div(pc))
    }
    .build
}
