package web.cps.a1quote

import ant.cps.Form.{ FormBackend, FormComponent, Item }
import ant.cps.FormCreate.TypedFieldOptions
import ant.cps._
import japgolly.scalajs.react.extra.router.RouterCtl
import japgolly.scalajs.react._
import japgolly.scalajs.react.vdom.html_<^._
import monix.eval.Task
import monix.execution.Ack
import monix.execution.cancelables.SingleAssignmentCancelable
import monix.reactive.Observer
import monix.reactive.subjects.PublishSubject
import nprogress.NProgress
import org.scalajs.dom
import relay.runtime.store.RelayEnvironment
import rmi.ReactMaskedInput
import shared.models.a1.{ HouseholdEnum, FieldKey => Fk }
import web.routes.Routes.{ Page, QuotePage }
import web.base.all._
import web.base.data.Attributable
import web.cps._
import web.mutation.QuoteA1BuilderM
import shared.proto.TaggedClassSerDe._
import shared.proto.a1.quote.NextStep

import scala.concurrent.Future
import scala.scalajs.js

object Step1 {

  case class Props(ctl: RouterCtl[Page],
                   page: QuotePage,
                   f: TypedFrag[QuoteA1Frag],
                   env: RelayEnvironment,
                   queue: PublishSubject[(Fk[_], TypedFieldOptions[js.Any])],
                   lgToMd: Boolean,
                   mdTo0: Boolean)

  class Backend(val $ : BackendScope[Props, Unit]) extends FormBackend {

    @inline def qa1: TypedFrag[QuoteA1Frag] = $.props.runNow().f
    val c: SingleAssignmentCancelable       = SingleAssignmentCancelable()

    // =================== Helpers ===============================
    def mount: Callback = $.props.flatMap { p =>
      //
      c := p.queue.subscribe(new Observer[(Fk[_], TypedFieldOptions[js.Any])] {
        def onNext(elem: (Fk[_], TypedFieldOptions[js.Any])): Future[Ack] = {
          val (fk, tfo) = elem
          QuoteParent.parseField(fk, tfo).fold[Future[Ack.Continue]](Ack.Continue) { rf =>
            NProgress.start()
            QuoteA1BuilderM.commit(p.env, rf, tfo).runAsync.flatMap { _ =>
              NProgress.done()
              // The result of each individual commit is captured in the `l` parameter
              Ack.Continue
            }
          }
        }
        def onError(ex: Throwable): Unit = ex.printStackTrace()
        def onComplete(): Unit           = println("Complete")
      })
      Callback.log("")
    }

    def unmount: Callback = Callback(c.cancel())

    val onSubmit = (e: ReactEvent) =>
      e.preventDefaultCB.toTask.flatMap { _ =>
        $.props.toTask.flatMap { p =>
          //
          validateFieldsT[js.Object]() flatMap {
            case Left(value) => Task(dom.console.error(value))
            case Right(value) =>
              Task(dom.console.log(value)) flatMap { _ =>
                QuoteA1BuilderM.commit(p.env, NextStep(2)) flatMap { _ =>
                  $.props.toTask
                    .map { p =>
                      if (p.f.r.step.get != 1) {
                        dom.console.log(s" SUB ${p.f.r.step}")
                        p.ctl.set(QuotePage(Some(2))).runNow()
                      } else ()
                    }
                }
              }
          }
        }
      }.runAsyncForget

    //
    // ===================== RENDER ME THIS.. =====================
    def render(p: Props) = {
      Form(layout = "horizontal", onSubmit = js.defined(onSubmit))(
        <.div(^.className := bareLo("header"))(
          <.h1("Welcome!"),
          <.p(
            "We're going to go over some details to give you the best, most accurate price." +
              "Only at the end will we ask for personal information.  Let us know if you have any " +
              "questions!"
          )
        ),
        FormCtl.ttrow()(
          <.div(
            "Let's make sure we can serve you.  Right now we can " +
              "only serve a limited amount of states."
          )
        ) { show =>
          FormMaskedInput.C(
            FormMaskedInput.Props(
              form,
              Fk.Zip.value,
              new ReactMaskedInput.Opts {
                override val mask        = "11111"
                override val placeholder = "5 digit zip"
              }.ex(show.toInputTagMod: _*),
              style.respItem("Enter your Zipcode")
            )
          )
        },
        FormCtl.ttrow()(
          <.div(
            "These correspond to exactly how you'd imagine they would.  Select the closest to your" +
              " situation if you have any doubt."
          )
        )(
          show =>
            Form.Item(
              style
                .respItem("Who's in your household?")
                .mergeLeftType(new js.Object {
                  val extra = "Choose what best describes your living situation. This helps gives " +
                    "us a feel for how much stuff you have.  You can always adjust later."
                  val width = "100%"
                })
            )(
              getFieldDecorator(Fk.Household.value)(
                Radio.Group.C(new Attributable {}.ex(show.toMouseTagMod :+ (^.width := "100%"): _*))(
                  Card(new Card.Opts {
                    override val noHovering = true
                  })(
                    Card.Grid(style = style.gridStyle)(
                      <.div(^.height := "64px", ^.marginBottom := "10px")(
                        <.img(^.src := "/assets/images/09_team_relationship_64.png")
                      ),
                      <.div(
                        Radio.Button(value = HouseholdEnum.MeOther.value)("Me & Roomates")
                      )
                    ),
                    Card.Grid(style = style.gridStyle)(
                      <.div(^.height := "64px", ^.marginBottom := "10px")(
                        <.img(^.src := "/assets/images/31_employer_avatar_64.png"),
                      ),
                      <.div(
                        Radio.Button(value = HouseholdEnum.Me.value)("Just Me")
                      )
                    ),
                    Card.Grid(style = style.gridStyle)(
                      <.div(^.height := "64px", ^.marginBottom := "10px")(
                        <.img(^.src := "/assets/images/05_workgroup_people_64.png"),
                      ),
                      <.div(
                        Radio.Button(value = HouseholdEnum.Family.value)("Me & Family")
                      )
                    ),
                  )
                )
              )
          )
        ),
        FormCtl.TTRow(
          FormCtl.TTRowP(
            <.div(
              "You can think of 1 star as mostly budget oriented furniture and affects, all the way to 5 star which " +
                "represents a curated, luxury collection."
            ), { show =>
              Form.Item(
                style
                  .respItem("How expensive is your stuff?")
                  .mergeLeftType(new Form.Item.Opt {
                    override val extra = "Think of this like question like Yelp, it gives us a better idea " +
                      "during replacement how much your stuff will cost.  Pick something and you can adjust this later."
                  })
                  .ex(show.toMouseTagMod: _*)
              )(
                getFieldDecorator(Fk.ContentsQuality.value)(
                  Rate(Rate.Opt)() //
                )
              )
            }
          )
        ),
        FormCtl.row()(
          Form.Item(style.respTailItem)(
            Button(htmlType = "submit", `type` = "primary", className = "jc-button")("Next Step")
          )
        )
      )

    }
  }

  object style {
    val gridStyle = new js.Object {
      val padding   = "12px"
      val textAlign = "center"
    }

    def respItem(s: String): Item.Opt = web.cps.style.respItem(s)
    def respTailItem: Item.Opt        = web.cps.style.respTailItem
    def fullItem(s: String): Item.Opt = new Item.Opt {
      override val label    = s
      override val required = true
    }
  }

  val C = ScalaComponent
    .builder[Props]("Step1")
    .stateless
    .renderBackend[Backend]
    .componentWillMount(_.backend.mount)
    .componentWillUnmount(_.backend.unmount)
    .build

  val FormC = FormCreate
    .builder[js.Dictionary[TypedFieldOptions[js.Any]]](C)
    .onFieldsChange { (p, f) =>
      f.toVector.flatMap(key => Fk.withValueOpt(key._1) map (_ -> key._2)) foreach { v =>
        p.unbox.queue.onNext(v)
      }
    //
    }
    .mapPropsToFields { p =>
      js.Dictionary(
        Seq(
          p.unbox.f.pair(Fk.ContentsQuality),
          p.unbox.f.pair(Fk.Zip),
          p.unbox.f.pairConvert(Fk.Household)(_.value)
        ): _*
      )
    }
    .build
}
