package web.cps.a1quote

import ant.cps.Form.{FormBackend, FormComponent, FormFacade}
import ant.cps._
import ant.cps.FormCreate.TypedFieldOptions
import japgolly.scalajs.react._
import japgolly.scalajs.react.extra.router.RouterCtl
import japgolly.scalajs.react.raw.ReactNode
import japgolly.scalajs.react.vdom.TagOf
import monix.eval.Task
import monix.execution.Ack
import monix.execution.cancelables.SingleAssignmentCancelable
import monix.reactive.Observer
import monix.reactive.subjects.PublishSubject
import nprogress.NProgress
import org.scalajs.dom
import org.scalajs.dom.html.Div
import relay.runtime.store.RelayEnvironment
import rmi.ReactMaskedInput
import vdom.html_<^._
import shared.models.a1.{DataCache, GuardTheftEnum, HouseholdEnum, FieldKey => Fk}
import shared.proto.a1.quote.NextStep
import web.base.all._
import web.cps.{bareLo, BareCard, DiscreteNumInput, SteppedNumInput}
import web.cps.DiscreteNumInput.formatUSD
import web.cps.a1quote.Step1.style
import web.mutation.QuoteA1BuilderM
import web.routes.Routes.{Page, QuotePage}
import shared.proto.TaggedClassSerDe._

import scala.concurrent.Future
import scala.scalajs.js
import scala.scalajs.js.UndefOr

object Step2 {

  case class Props(ctl: RouterCtl[Page],
                   page: QuotePage,
                   f: TypedFrag[QuoteA1Frag],
                   env: RelayEnvironment,
                   queue: PublishSubject[(Fk[_], TypedFieldOptions[js.Any])],
                   lgToMd: Boolean,
                   mdTo0: Boolean)

  val alarmEnum: ReactNode = <.div(
    <.p("Here's how to think about these options"),
    <.ul(
      <.li(<.strong("Local"), " means an alarm that just makes noise"),
      <.li(<.strong("Direct"), " means it contacts you"),
      <.li(<.strong("Central"), " means it rings a service to respond to the fire/theft."))).rawNode

  val cardOpt: Card.Opts = new Card.Opts {
    override val bodyStyle = (^.padding := "0px").toJs.styles
  }

  class Backend(val $ : BackendScope[Props, Unit]) extends FormBackend {

    val c: SingleAssignmentCancelable       = SingleAssignmentCancelable()
    @inline def qa1: TypedFrag[QuoteA1Frag] = $.props.runNow().f

    // ===================== Lifecycle ==============================
    def mount: Callback =
      $.props.map { p =>
        c := p.queue.subscribe(new Observer[(Fk[_], TypedFieldOptions[js.Any])] {
          def onNext(elem: (Fk[_], TypedFieldOptions[js.Any])): Future[Ack] = {
            val (fk, tfo) = elem
            QuoteParent.parseField(fk, tfo).fold[Future[Ack.Continue]](Ack.Continue) { rf =>
              NProgress.start()
              QuoteA1BuilderM.commit(p.env, rf, tfo).runAsync.flatMap { _ =>
                NProgress.done()
                // The result of each individual commit is captured in the `l` parameter
                Ack.Continue
              }
            }
          }
          def onError(ex: Throwable): Unit = ex.printStackTrace()
          def onComplete(): Unit           = println("Complete")
        })
      }.void

    def unmount: Callback = Callback(c.cancel())

    // ====================== Event Helpers ==========================
    val goBack: UndefOr[js.Function1[ReactEvent, Unit]] =
      ((_: ReactEvent) => $.props.flatMap(_.ctl.set(QuotePage(Some(1))))).toJsCallback

    val submit: (ReactEvent) => Callback = (ev: ReactEvent) =>
      ev.preventDefaultCB >> $.props.map { props =>
        val r = for {
          cb <- formSubmitTrigger()
          vf <- validateFieldsAndScrollT[js.Object]()
          _ <- vf match {
            case Left(value) => Task.unit
            case Right(value) =>
              NProgress.start()
              QuoteA1BuilderM.commit(props.env, NextStep(3)) flatMap { _ =>
                $.props.toTask
                  .flatMap { p =>
                    NProgress.done()
                    cb()
                    if (p.f.r.step.get != 2) {
                      dom.console.log(s" SUB ${p.f.r.step}")
                      p.ctl.set(QuotePage(Some(3))).toTask
                    } else Task.unit
                  }
              }
          }
        } yield {}
        r.runAsyncForget
      }.void

    def render(p: Props) = {
      Form(new Form.Opt {
        override val onSubmit = submit.toJsCallback
      })(
        <.div(^.className := bareLo("header"))(
          <.h1("We've picked some smart defaults"),
          <.p("""
              |Please adjust these as you see fit. We'll try to explain everything along the way. However,
              |keep in mind that this is all adjustable (and should be!) at any point in time and we'll pro-rate
              |everything so you are never paying a cent more than you should.
            """.stripMargin)),
        FormCtl.ttrow()(
          <.div(
            <.p("Pay attention to this as it will change your price and your loss repayment."),
            <.p(<.strong("250 - "), <.span("Higher monthly price but best outcome during a loss.")),
            <.p(
              <.strong("2,500 "),
              <.span("- This should be used only for large contents coverage where a total loss " +
                "is your primary concern."))))(
          show =>
            <.div(show.toMouseTagMod: _*)(
              Form.Item(style
                .respItem("Deductible")
                .mergeLeftType(new Form.Item.Opt {
                  override val extra = <.div(
                    <.p(
                      "Deductible determines the amount of money that gets " +
                        "deducted from a claim in the event of a loss.  For example, if you have a $1000" +
                        " valid claim and a $250 deductible we'd pay you 1000-250 = $750."),
                    <.div(^.className := "recommend")(s"recommended: ${qa1.rec(Fk.Deduct).map(formatUSD)}")).rawNode
                }))(getFieldDecorator(Fk.Deduct, new FormComponent.Opt {
                override val rules = js.defined(js.Array(new FormComponent.RuleOpt {
                  override val message = "Please input your choice"
                  val required         = true
                }))
              })(Radio.Group()(DataCache.deductible.toVdomArray(f =>
                Radio.Button(value = f, key = f.toString)(formatUSD(f)))))))),
        FormCtl.ttrow()(
          <.ul(
            <.li(<.strong("Amazon (Replacement Cost)"), <.span(" Better claiming experience, but costs more")),
            <.li(<.strong("Craigslist (Actual Cash Value)"), <.span(" More affordable policy.")))) { show =>
          <.div(show.toMouseTagMod: _*)(
            Form.Item(
              style
                .respItem("Replacement strategy")
                .mergeLeftType(new Form.Item.Opt {
                  override val extra =
                    <.div(
                      <.strong("We typically recommend going with Replacement cost, or the option 'Amazon'."),
                      <.p(
                        """ If you were going to replace your couch and it cost $1000, we'd give you $1000 minus
                          | your deductible.  With the actual cash value approach, or 'Craigslist', we'd
                          | calculate that by factoring in what you could purchase your item for today, used, so a 3 year
                          | old couch might get $600 instead.
                    """.stripMargin)).rawNode
                }))(getFieldDecorator(Fk.Replacement, new FormComponent.Opt {
              override val rules = js.defined(js.Array(new FormComponent.RuleOpt {
                override val message = "Please select one"
                val required         = true
              }))
            })(Radio.Group()(Radio.Button(value = false)("Craigslist"), Radio.Button(value = true)("Amazon")))))
        },
        FormCtl
          .splitTTRow()(<.div("Someone else got hurt"), <.div("Right"))(
            { s =>
              Card(new Card.Opts {
                override val bodyStyle = (^.padding := "0px").toJs.styles
              })(
                BareCard.C(
                  BareCard.Props(
                    "Liability Coverage",
                    <.img(^.src := "/assets/images/07_auction_hammer_128.png"),
                    "I am held liable for some negligence"))(
                  <.p(
                    "This protects against accidental negligence.  Which in insurance speak means if something " +
                      "happened at your insured location, that was not intentional but somebody got 'hurt' (which " +
                      "also has a complex definition), Goodcover would cover it."),
                  DiscreteNumInput.C(DiscreteNumInput.Props(
                    form,
                    Fk.CovE.value,
                    DataCache.coverageE.head,
                    DataCache.coverageE.last,
                    DataCache.coverageE,
                    DataCache.coverageE.head,
                    fmt     = formatUSD,
                    tagMods = Seq(^.marginTop := "10px"))),
                  <.div(^.className := "recommend")(s"recommended: ${qa1.recOpt(Fk.CovE).map(formatUSD)}")))
            }, { s =>
              Card(new Card.Opts {
                override val bodyStyle = (^.padding := "0px").toJs.styles
              })(
                BareCard.C(
                  BareCard.Props(
                    "Property Coverage",
                    <.img(^.src := "/assets/images/07_mobile_banking_128.png"),
                    "My stuff was stolen/damaged"))(
                  <.p("This protects against theft and loss. So if someone breaks in and steals your property " +
                    "Goodcover will make you whole (That is after all the point of insurance)."),
                  DiscreteNumInput.C(DiscreteNumInput.Props(
                    form,
                    Fk.CovC.value,
                    DataCache.coverageC.head,
                    DataCache.coverageC.last,
                    DataCache.coverageC,
                    DataCache.coverageC.head,
                    fmt     = formatUSD,
                    tagMods = Seq(^.marginTop := "10px"))),
                  <.div(^.className := "recommend")(s"recommended: ${qa1.rec(Fk.CovC).map(formatUSD)}")))
            }),
        //
        FormCtl.splitTTRow()(<.div("Left"), <.div("Right"))(
          s =>
            Card(cardOpt)(
              BareCard.C(
                BareCard.Props(
                  "Loss of Use",
                  <.img(^.src := "/assets/images/13_location_mark_128.png"),
                  "My place is uninhabitable"))(
                <.p("Loss of use coverage protects you when you can comfortably live in your apartment/home " +
                  "due to something unforeseen."),
                <.p("We cover a default amount of loss of use, any amount would ", <.strong("be in addition")),
                <.h2(^.textAlign.center)(" + " + qa1(Fk.CovDDefault).map(formatUSD).getOrElse[String]("N/A")),
                SteppedNumInput.C(
                  SteppedNumInput.Props(form, Fk.CovD, _ + 1000, _ - 1000, 0, max = 20000, fmt = formatUSD)),
                <.h2(^.textAlign.center)(" = " + qa1(Fk.CovDLimit).map(formatUSD).getOrElse[String]("N/A")),
                <.div(^.className := "recommend")(s"recommended additional: ${qa1.recOpt(Fk.CovD).map(formatUSD)}"))),
          s =>
            Card(cardOpt)(
              BareCard.C(
                BareCard.Props(
                  "Medical Payments to Others",
                  <.img(^.src := "/assets/images/06_employee_insurance_128.png"),
                  "Someone got injured"))(
                <.p("This covers small claims type injuries, basically it avoids having to go to court over" +
                  " small injuries, by immediately paying the victim."),
                DiscreteNumInput.C(DiscreteNumInput.Props(
                  form,
                  Fk.CovF.value,
                  DataCache.coverageF.head,
                  DataCache.coverageF.last,
                  DataCache.coverageF,
                  DataCache.coverageF.head,
                  fmt     = formatUSD,
                  tagMods = Seq(^.marginTop := "10px")))))),
        propertyDetails,
        FormCtl.row(^.textAlign.center)(
          Row(`type` = "flex", justify = "space-around")(
            Button(onClick = goBack)(Icon("left"), "Go Back"),
            Button(`type`  = "primary", htmlType = "submit")("Next Step, Whoosh", Icon("right")))))
    }

    def propertyDetails: TagOf[Div] = {
      <.div(
        <.div(^.className := bareLo("header"))(
          <.h1("Property Details and Discounts!"),
          <.p("""
              | Select the answer that best describes some details about your property.  This helps
              | us give you a price that's closer to your risk level.
            """.stripMargin)),
        FormCtl.ttrow()(<.div("Swimming pools cause liability issues, even in an apartment complex."))(
          show =>
            <.div(show.toMouseTagMod: _*)(Form.Item(style
              .respItem("Do you have a swimming pool?")
              .mergeLeftType(new Form.Item.Opt {
                override val extra = "We break out swimming pools as to not subsidize the folks with swimming pools. " +
                  "We think it delivers a more fair price."
              }))(getFieldDecorator(Fk.SwimmingP)(
              Radio.Group()(Radio.Button(value = false)("No"), Radio.Button(value = true)("Yes")))))),
        FormCtl.ttrow()(
          <.div(
            "Bigger buildings have less chance of theft, since more people will be present at the insured location."))(
          show =>
            <.div(show.toMouseTagMod: _*)(
              Form.Item(style
                .respItem("How many stories do you have?")
                .mergeLeftType(new Form.Item.Opt {
                  override val extra = "This helps us determine the size of the building."
                }))(getFieldDecorator(Fk.NumStories.value)(Radio.Group()(DataCache.numStories.toVdomArray(f =>
                Radio.Button(value = f, key = f)(f))))))),
        FormCtl.splitTTRow()(
          <.div("Let us know any preventive fire devices you might have."),
          <.div("Let us know any preventive theft devices you might have."))(
          s =>
            Card(cardOpt.ex(s.toMouseTagMod: _*))(BareCard.C(
              BareCard.Props(None, <.img(^.src := "/assets/images/36_antivirus_firewall_128.png"), "Fire Protection"))(
              Form.Item(style.fullItem("Fire Extinguisher?"))(getFieldDecorator(Fk.FireExt)(
                Radio.Group()(Radio.Button(value = false)("No"), Radio.Button(value = true)("Yes")))),
              Form.Item(style.fullItem("Smoke Alarm?"))(getFieldDecorator(Fk.SmokeAlarm)(
                Radio.Group()(Radio.Button(value = false)("No"), Radio.Button(value = true)("Yes")))),
              Form.Item(style
                .fullItem("Fire Alarm?")
                .mergeLeftType(new Form.Item.Opt {
                  override val extra = alarmEnum
                }))(
                //
                getFieldDecorator(Fk.FireAlarm)(
                  Radio.Group()(
                    Radio.Button(value = "No")("No"),
                    Radio.Button(value = "Local")("Local"),
                    Radio.Button(value = "Direct")("Direct"),
                    Radio.Button(value = "Central")("Central"),
                  ))),
              Form.Item(style.fullItem("Sprinkler in your unit?"))(
                //
                getFieldDecorator(Fk.Sprinkler)(Radio.Group()(
                  Radio.Button(value = "No")("No"),
                  Radio.Button(value = "Partial")("Partial"),
                  Radio.Button(value = "Full")("Full")))),
            )),
          s =>
            Card(cardOpt.ex(s.toMouseTagMod: _*))(
              BareCard.C(
                BareCard.Props(None, <.img(^.src := "/assets/images/16_security_lock_128.png"), "Theft protection, "))(
                Form.Item(style.fullItem("Deadbolt?"))(getFieldDecorator(Fk.Deadbolt)(
                  Radio.Group()(Radio.Button(value = false)("No"), Radio.Button(value = true)("Yes")))),
                Form.Item(
                  style
                    .fullItem("Theft alarm?")
                    .mergeLeftType(new Form.Item.Opt {
                      override val extra = alarmEnum
                    }))(
                  getFieldDecorator(Fk.TheftAlarm)(
                    Radio.Group()(
                      Radio.Button(value = "No")("No"),
                      Radio.Button(value = "Local")("Local"),
                      Radio.Button(value = "Direct")("Direct"),
                      Radio.Button(value = "Central")("Central"),
                    ))),
                Form.Item(style.fullItem("Do you have a Guard?"))(
                  //
                  getFieldDecorator(Fk.Guard)(Radio.Group()(
                    Radio.Button(value = GuardTheftEnum.No.value)("No"),
                    Radio.Button(value = GuardTheftEnum.LimitedAccess.value)("Gated Community"),
                    Radio.Button(value = GuardTheftEnum.SelfEnclosed.value)("Doorman"),
                  ))),
              ))))
    }
  }

  val C = ScalaComponent
    .builder[Props]("Step2")
    .stateless
    .renderBackend[Backend]
    .componentDidMount(_.backend.mount)
    .componentWillUnmount(_.backend.unmount)
    .build

  val FormC = FormCreate
    .defaultBuilder(C)
    .onFieldsChange { (p, f) =>
      f.toVector.flatMap(key => Fk.withValueOpt(key._1) map (_ -> key._2)) foreach { v =>
        p.unbox.queue.onNext(v)
      }
    }
    .mapPropsToFields { p =>
      js.Dictionary(
        Seq(
          /* Form Ordering-ish */
          p.unbox.f.pair(Fk.CovC),
          p.unbox.f.pairOpt(Fk.CovD),
          p.unbox.f.pairOpt(Fk.CovE),
          p.unbox.f.pairOpt(Fk.CovF),
          p.unbox.f.pair(Fk.Replacement),
          p.unbox.f.pair(Fk.Deduct),
          p.unbox.f.pair(Fk.SwimmingP),
          p.unbox.f.pair(Fk.NumStories),
          /* Discounts */
          p.unbox.f.pair(Fk.FireExt),
          p.unbox.f.pair(Fk.SmokeAlarm),
          p.unbox.f.pairConvert(Fk.FireAlarm)(_.value),
          p.unbox.f.pairConvert(Fk.Sprinkler)(_.value),
          p.unbox.f.pair(Fk.Deadbolt),
          p.unbox.f.pairConvert(Fk.TheftAlarm)(_.value),
          p.unbox.f.pairConvert(Fk.Guard)(_.value),
        ): _*)
//      js.Dictionary()
    }
    .build
}
