package web.cps.a1quote

import ant.cps.Form.{FormBackend, FormFacade}
import ant.cps.FormCreate.TypedFieldOptions
import ant.cps._
import japgolly.scalajs.react._
import japgolly.scalajs.react.vdom.html_<^._
import monix.eval.Task
import monix.execution.Ack
import monix.execution.cancelables.SingleAssignmentCancelable
import monix.reactive.Observer
import monix.reactive.subjects.PublishSubject
import nprogress.NProgress
import org.scalajs.dom
import relay.runtime.store.RelayEnvironment
import shared.models.a1.{FieldKey => Fk}
import shared.proto.TaggedClassSerDe._
import shared.proto.a1.quote.NextStep
import web.cps.a1quote.Step1.style
import web.base.all._
import web.cps.bareLo
import web.mutation.QuoteA1BuilderM
import web.routes.Routes.QuotePage

import scala.concurrent.Future
import scala.scalajs.js

object Step3 {
  //
  case class Props(qp: QuoteParent.Props,
                   f: TypedFrag[QuoteA1Frag],
                   env: RelayEnvironment,
                   queue: PublishSubject[(Fk[_], TypedFieldOptions[js.Any])],
                   lgToMd: Boolean,
                   mdTo0: Boolean)

  val C = ScalaComponent
    .builder[Props]("Step3")
    .stateless
    .renderBackend[Backend]
    .componentDidMount(_.backend.mount)
    .componentWillUnmount(_.backend.unmount)
    .build

  val FormC = FormCreate
    .defaultBuilder(C)
    .onFieldsChange((p, f) =>
      f.toVector.flatMap(key => Fk.withValueOpt(key._1) map (_ -> key._2)) foreach { v =>
        p.unbox.queue.onNext(v)
    })
    .mapPropsToFields { p =>
      //
      js.Dictionary(
        Seq(
          p.unbox.f.pair(Fk.SecuritiesSL),
          p.unbox.f.pair(Fk.JewelrySL),
          p.unbox.f.pair(Fk.MoneySL),
          p.unbox.f.pair(Fk.SilverwareSL),
          p.unbox.f.pair(Fk.FirearmsSL),
          p.unbox.f.pair(Fk.ElectronicApparatusSL),
          p.unbox.f.pair(Fk.TapesRecordsSL),
          p.unbox.f.pair(Fk.PootersSL),
        ): _*)
    }
    .build

  class Backend(val $ : BackendScope[Props, Unit]) extends FormBackend {

    val c: SingleAssignmentCancelable = SingleAssignmentCancelable()

    // =================== Helpers ===============================
    def mount: Callback = $.props.flatMap { p =>
      //
      c := p.queue.subscribe(new Observer[(Fk[_], TypedFieldOptions[js.Any])] {
        def onNext(elem: (Fk[_], TypedFieldOptions[js.Any])): Future[Ack] = {
          val (fk, tfo) = elem
          QuoteParent.parseField(fk, tfo).fold[Future[Ack.Continue]](Ack.Continue) { rf =>
            NProgress.start()
            QuoteA1BuilderM.commit(p.env, rf, tfo).runAsync.flatMap { _ =>
              NProgress.done()
              // The result of each individual commit is captured in the `l` parameter
              Ack.Continue
            }
          }
        }
        def onError(ex: Throwable): Unit = ex.printStackTrace()
        def onComplete(): Unit           = println("Complete")
      })
      Callback.log("")
    }

    def unmount: Callback = Callback(c.cancel())

    def goBack = $.props.flatMap(p => p.qp.ctl.set(QuotePage(Some(2))))

    val submit = (r: ReactEvent) =>
      r.preventDefaultCB.map(_ =>
        (for {
          _ <- Task.pure(NProgress.start())
          pa <- $.props.toTask
          cb <- formSubmitTrigger()
          r <- validateFieldsAndScrollT[js.Object]()
          _ <- r match {
            case Left(value) => Task.unit
            case Right(value) =>
              QuoteA1BuilderM.commit(pa.env, NextStep(4)) flatMap { _ =>
                cb()
                $.props.toTask flatMap { pb =>
                  if (pb.f.r.step.get >= 4) {
                    dom.console.log(s" SUB ${pb.f.r.step}")
                    pb.qp.ctl.set(QuotePage(Some(4))).toTask
                  } else Task.unit
                }
              }

          }
          _ <- Task.pure(NProgress.done())
        } yield ()).runAsyncForget)

    //
    def render =
      Form(onSubmit = submit.toJsCallback)(
        <.div(^.className := bareLo("header"))(
          <.h1("Special Items"),
          <.h2("This is optional adjust as needed but the defaults are a good mix for most people."),
          <.p(
            <.span("""
                   |Some items are more prone to problems and those are covered under a different type
                   |of arrangement, and they are what we call
                   |
            """.stripMargin),
            <.strong("Special Items."),
            <.span("""
                   |You can always change these items later, but if you know what you'd like, you can
                   | change them now. Keep in mind, these limits are subject to different deductibles.
            """.stripMargin))),
        FormCtl.row()(SpecialItemsCtl.C(SpecialItemsCtl.Props(form))),
        FormCtl.row(^.textAlign.center)(
          Row(`type` = "flex", justify = "space-around")(
            Button(onClick = goBack.toJsFn1)(Icon("left"), "Go Back"),
            Button(`type`  = "primary", htmlType = "submit")("Next Step, Almost there.", Icon("right")))))
  }
}
