package web.cps.chat

import ant.cps.{Button, Input}
import japgolly.scalajs.react._
import japgolly.scalajs.react.vdom.html_<^._
import monix.execution.Ack
import monix.execution.cancelables.SingleAssignmentCancelable
import monix.reactive.Observer
import monix.reactive.subjects.PublishSubject
import org.scalajs.dom
import org.scalajs.dom.html
import web.base.all._
import web.cps.chatLo

import scala.concurrent.Future
import scala.concurrent.duration._
import scala.scalajs.js
import scala.scalajs.js.JSConverters._

object ChatInterface {
  case class Props(queue: PublishSubject[ChatGrammar],
                   msgs: js.Array[ChatGrammar],
                   onSend: Callback,
                   onChange: String => Callback,
                   value: Option[String])
  case class State(typing: Boolean, text: String) {}

  private val rawEmpty = raw"\u00a0"

  val C = ScalaComponent
    .builder[Props]("ChatInterface")
    .initialState(State(false, ""))
    .renderBackend[Backend]
    .componentDidMount(_.backend.didMount)
    .build

  private def lchat(key: String, s: VdomNode) =
    <.div(^.className := chatLo("cb"), ^.className := "left", ^.key := key)(<.p(s))

  private def rchat(key: String, s: VdomNode) =
    <.div(^.className := chatLo("cb"), ^.className := "right", ^.key := key)(<.p(s))

  class Backend(val $ : BackendScope[Props, State]) {
    private var outerRef: html.Element = _
    val c                              = SingleAssignmentCancelable()

    def didMount: Callback = scrollToContainer >> $.props.map { p =>
      c := p.queue
        .bufferTimed(750.millis)
        .filter(_.nonEmpty)
        .distinct
        .subscribe(new Observer[Seq[ChatGrammar]] {
          def onNext(elem: Seq[ChatGrammar]): Future[Ack] = {
            Future
              .traverse(elem) {
                case ScrollBottom => Future.successful(scrollToContainer.runNow())
                case _            => Future.unit
              }
              .map(_ => Ack.Continue)
          }
          def onError(ex: Throwable): Unit = ex.printStackTrace()
          def onComplete(): Unit           = println("Complete")
        })
      ()
    }

    def scrollToContainer = Callback { outerRef.scrollTop = outerRef.scrollHeight.toDouble }

    def onChange(f: ReactEventFromInput): CallbackTo[Unit] = $.props.flatMap { p =>
      p.onChange(f.target.value)
    }

    def render(p: Props, s: State) = {
      <.div(^.className := chatLo("wrap"))(
        <.div(^.className := chatLo("container")).ref(outerRef = _)(p.msgs.collect {
          case Msg(id, None, msg, ts) =>
            lchat(id.toString, msg)

          case Msg(id, Some(who), msg, ts) =>
            rchat(id.toString, msg)
        }.toVdomArray),
        <.div(^.className := chatLo("trigger"))(rawEmpty),
        <.div(^.className := chatLo("form"))(Input(new Input.Opt {
          override val placeholder  = "Type here"
          override val value        = p.value.orUndefined
          override val onPressEnter = p.onSend.toJsFn1
        }.ex(^.onChange ==> onChange)), Button(new Button.Opt {
          override val `type` = "primary"
        }.ex(^.onClick --> p.onSend))("Send")))
    }
  }
}
