package web.cps.claim

import ant.cps._
import dispalt.relay.gql
import japgolly.scalajs.react._
import japgolly.scalajs.react.component.Scala
import japgolly.scalajs.react.component.builder.Lifecycle.ComponentWillReceiveProps
import japgolly.scalajs.react.vdom.html_<^._
import monix.execution.Ack
import monix.execution.cancelables.{MultiAssignmentCancelable, SingleAssignmentCancelable}
import monix.reactive.{Observer, OverflowStrategy}
import monix.reactive.subjects.PublishSubject
import org.scalajs.dom
import web.react.{RelayBackend, RelayBackendRefetch}
import relay.components._
import relay.generated.{ClaimDetailView, ClaimDetail_claim, ClaimDetail_details, ClaimDetail_viewer}
import relay.graphql.TaggedNode
import relay.runtime.{RecordSourceSelectorProxy, StoreDebugger}
import relay.runtime.RelayRuntime.ConnectionHandler
import relay.runtime.subscription.{RequestRelaySubscription, RequestRelayVars}
import shared.models.GId
import shared.proto.claims.UserUtter
import web.base.data.{BaseViewerFrag, SangriaConnection, SangriaEdge}
import web.cps.bareLo
import web.routes.Routes
import web.views.Core
import web.base.all._
import web.cps.chat.{ChatGrammar, ChatInterface, Msg, ScrollBottom}
import web.cps.claim.ClaimView.queryDetail
import web.mutation.ClaimBuilderMutation

import scala.concurrent.Future
import scala.scalajs.js

object ClaimDetail {

  /*
  type ClaimChatEvent {
  # The ID of an object
  id: ID!
  rawId: String!
  user: UserUtterType
  bot: BotUtterType
  sig: ClaimBotOp
}
   */

  @js.native
  trait ClaimChatEventUserUtter extends js.Object {
    val msg: String
    val who: OrNull[String]
    val when: Double
  }

  @js.native
  trait ClaimChatEventBotUtter extends js.Object {
    val msg: String
    val when: Double
  }

  @js.native
  trait ClaimChatEvent extends js.Object {
    val id: String
    val user: OrNull[ClaimChatEventUserUtter]
    val bot: OrNull[ClaimChatEventBotUtter]
  }

  @js.native
  trait Details extends js.Object {
    val id: String
    val status: String
    val lastId: Double
    val messages: SangriaConnection[ClaimChatEvent]
    val nextQuestion: OrNull[NextQuestion]
  }

  @js.native
  trait NextQuestion extends js.Object {
    val slotKey: String
    val question: String
    val confirmation: Boolean
    val select: Boolean
    val prompt: OrNull[String]
  }

  trait ClaimProcess extends js.Object {
//    val details: Details
    val claimId: String
    val summary: String
    val timeCreatedMs: Double
  }

  trait Foo extends js.Object {
    val claim: ClaimProcess
    val details: Details
  }

  //
  @gql("""
      fragment ClaimDetail_viewer on Viewer {
        id
        ...Core_viewer
      }
   """)
  object frag extends TaggedNode

  @gql("""
      fragment ClaimDetail_claim on Claim {
        claimId status summary timeCreatedMs
        details {
          ...ClaimDetail_details
        }
      }
      """)
  object claimDetailClaim extends TaggedNode

  @gql("""
      fragment ClaimDetail_details on ClaimInteraction {
        id status lastId

        messages(first: $first) @connection(key: "ClaimDetail_messages") {
          edges {
            node {
              id __typename
              user {
                msg who when
              }
              bot {
                msg when
              }
            }
            cursor
          }
        }

        nextQuestion {
          slotKey
          question
          confirmation
          select
          prompt
        }
      }
      """)
  object claimDetailDetails extends TaggedNode

  @gql("""
      subscription ClaimDetail_sub($claimId: String!, $startId: Long!) {
        claimMessages(claimId: $claimId, startId: $startId) {
          id __typename
          sig {
            __typename
            ... on ClaimBotOpTyping {
              duration
            }
          }
          user {
            msg who when
          }
          bot {
            msg when
          }
        }
      }
    """)
  object sub extends TaggedNode

  type RP    = BaseViewerFrag with Foo
  type Props = ClaimView.Props

  case class State(msg: Option[String] = None) {
    def setMsg(m: String) = copy(msg = Some(m))
  }

  val C = ScalaComponent
    .builder[ClaimView.Props]("ClaimDetail")
    .initialState(State())
    .renderBackend[Backend]
    .componentDidMount(_.backend.didMount)
    .componentWillMount(_.backend.willMount)
    .buildRelay[RP]

  val Container = RefetchContainer.buildRelay(C,
                                              ClaimDetailView,
                                              "viewer"  -> ClaimDetail_viewer,
                                              "claim"   -> ClaimDetail_claim,
                                              "details" -> ClaimDetail_details)

  val colOps = new Col.Opt {
    override val span = 12

    override val lg = 12
    override val md = 24
    override val sm = 24
    override val xs = 24
  }
  class Backend(val $ : BackendScope[ClaimView.Props, State]) extends RelayBackendRefetch[RP] {
    //
    private val queue = PublishSubject[ChatGrammar]()
    val cancelable    = MultiAssignmentCancelable()

    def willMount: Callback = Callback.log("didMount")
    def didMount: Callback =
      Callback {
        val nextId = relayProps.details.lastId + 1
        cancelable :=
          RequestRelaySubscription[js.Object](relay.environment,
                                              sub,
                                              new RequestRelayVars {
                                                val claimId = relayProps.claim.claimId
                                                val startId = nextId
                                              }, { store: RecordSourceSelectorProxy =>
                                                val details = store.get(relayProps.details.id)

                                                val conn = ConnectionHandler
                                                  .getConnection(details.get, "ClaimDetail_messages")
                                                  .get
                                                store.getRootField("claimMessages") foreach { node =>
                                                  val newEdge =
                                                    ConnectionHandler.createEdge(store,
                                                                                 conn,
                                                                                 node,
                                                                                 node.getValueStr("__typename"))
                                                  val cursor = node.getValueStr("id").toString
                                                  newEdge.setValue(cursor, "cursor")
                                                  ConnectionHandler.insertEdgeAfter(conn, newEdge, cursor)
                                                  queue.onNext(ScrollBottom)
                                                }
                                              })(OverflowStrategy.DropOld(100))
            .subscribe(new Observer[js.Object] {
              override def onError(ex: Throwable): Unit = ex.printStackTrace()

              override def onComplete(): Unit = {
                println("OnComplete, reconnecting")
                didMount.runNow()
              }

              override def onNext(elem: js.Object): Future[Ack] = {
                dom.console.log("onNext", elem)
                Ack.Continue
              }
            })
      } >> Callback.log("Mount")

    val refresh = Callback {
      //
      relay.refetch({ f: js.Dictionary[js.Any] =>
        dom.console.log(f)
        f
      }: js.Function1[js.Dictionary[js.Any], js.Dictionary[js.Any]])
    }

    def refreshBtn =
      Button(new Button.Opt {
        override val icon    = "reload"
        override val onClick = refresh.toJsFn1
      })()

    def onSend = $.state.map { s =>
      import shared.proto.TaggedClassSerDe._
      ClaimBuilderMutation
        .commit(relay.environment,
                UserUtter(ec.currentTimeMillis(), GId(relayProps.claim.claimId), s.msg.getOrElse("")),
                relayProps.viewer.id)
        .runAsyncForget
      ()
    }

    def onChange = (s: String) => $.modState(_.setMsg(s))

    def render(p: Props, s: State) = {
      dom.console.log(relayProps.details.messages)
      val msgs: js.Array[ChatGrammar] = relayProps.details.messages.edges.flatMap { f =>
        if (f.node.bot.nonEmpty) {
          val b = f.node.bot.get
          Seq[ChatGrammar](Msg(b.when.toLong, None, b.msg, b.when.toLong))
        } else if (f.node.user.nonEmpty) {
          val b = f.node.user.get
          Seq[ChatGrammar](Msg(b.when.toLong, b.who.toOption, b.msg, b.when.toLong))
        } else Seq.empty
      }

      dom.console.log(msgs)

      Core.Container
        .rprops(relayProps)(Core.Props(true, p.ctl, Routes.ClaimsPage(None)))(
          Row(`type` = "flex", gutter = 16)(
            Col(colOps)(Card(new Card.Opts {
              override val className  = "card-thck"
              override val noHovering = true
              override val title =
                <.span(<.span("Claim"), <.span(" - "), <.code(<.strong(relayProps.claim.claimId))).rawElement
              override val extra = refreshBtn.rawElement
            }.ex(^.marginBottom := "20px"))(ChatInterface.C(
              ChatInterface.Props(queue, msgs, onSend, onChange, s.msg)))),
            Col(colOps)(Card(new Card.Opts {
              override val className  = "card-thck"
              override val noHovering = true
            }.ex(^.marginBottom := "20px"))(<.h2("Claim Details"),
                                            row("Status", relayProps.details.status),
                                            relayProps.details.nextQuestion.fold(VdomNode.cast(null)) { nq =>
                                              <.div(row("Pending question", nq.question))
                                            }))))
    }
  }

  def row(name: String, value: VdomNode) = {
    Row()(Col(new Col.Opt {
      override val span = 12
    })(name), Col(new Col.Opt {
      override val span = 12
    })(value))
  }
//    .
}
