package web.mutation

import dispalt.relay.gql
import monix.eval.Task
import monix.execution.CancelableFuture
import relay.generated.{ClaimBuilder, ClaimStartMutation}
import relay.graphql.TaggedNode
import relay.runtime.RelayRuntime.{ConnectionHandler, ViewerHandler}
import relay.runtime.mutation.CommitMutation
import relay.runtime.store.RelayEnvironment
import shared.proto.TaggedClass
import web.base.data.{BaseId, ClaimFrag, KeyedRateFailure, SangriaEdge}
import web.base.all._

import scala.scalajs.js
import scala.scalajs.js.|

object ClaimBuilderMutation {

  trait Result extends js.Object

  trait StartResultClaim extends js.Object {
    val claimEdge: OrNull[SangriaEdge[ClaimFrag]]
    val error: OrNull[String]
  }
  trait StartResult extends js.Object {
    val claimStart: StartResultClaim
  }

  @gql("""
    mutation ClaimBuilder($input: ClaimProcessBuilderInput!) {
      claimBuilder(input: $input) {
        clientMutationId
        resp {
          id status lastId
          nextQuestion {
            slotKey
            question
            confirmation
            select
            prompt
          }
        }
      }
    }
  """)
  object mut extends TaggedNode

  @gql("""
    mutation ClaimStartMutation($input: ClaimStartInput!) {
      claimStart(input: $input) {
        clientMutationId
        claimEdge {
          __typename
          cursor
          node {
            id rawId
            claimId
            summary status timeCreatedMs
          }
        }
      }
    }
  """)
  object claimStartMutation extends TaggedNode

  claimStartMutation //

  def commit[M: TaggedClass](env: RelayEnvironment,
                             msg: M,
                             viewerId: String): Task[Vector[KeyedRateFailure] Either Result | Null] = {
    CommitMutation[PBufInput, Result](env, ClaimBuilder, PBufInput(msg)) map {
      case Left(e) =>
        Left(Vector(KeyedRateFailure("", e.map(_.message))))
      case Right(result) =>
        Right(result)
    }
  }

  def start[M: TaggedClass](env: RelayEnvironment,
                            msg: M,
                            viewerId: String): Task[Vector[KeyedRateFailure] Either StartResult | Null] = {
    CommitMutation[PBufInput, StartResult](env, relay.generated.ClaimStartMutation, PBufInput(msg), update = Some {
      rsp =>
        for {
          viewer  <- rsp.get(viewerId)
          payload <- rsp.getRootField("claimStart")
          newEdge <- payload.getLinkedRecord("claimEdge")
          conn    <- ConnectionHandler.getConnection(viewer, "ClaimView_claims")
        } yield ConnectionHandler.insertEdgeBefore(conn, newEdge)
        ()
    }) map {
      case Left(e) =>
        Left(Vector(KeyedRateFailure("", e.map(_.message))))
      case Right(result) =>
        Right(result)
    }
  }
}
