package web.routes

import ant.cps.{Alert, LocaleProvider}
import dispalt.relay.gql
import japgolly.scalajs.react._
import relay.components._
import japgolly.scalajs.react.extra.router._
import japgolly.scalajs.react.component.Generic._
import japgolly.scalajs.react.vdom.html_<^._
import org.scalajs.dom
import relay.QueryRenderer
import relay.graphql.TaggedNode
import web.base.data.{BaseViewerFrag, LocalRelayError}
import web.base.all._
import web.cps.a1quote.{QuoteA1View, QuoteParent}
import web.cps.claim.ClaimView
import web.cps.{LoggedInSpinner, PageSpinner}
import web.run.Env
import web.views
import web.views.Lander

import scala.scalajs.js
import scala.scalajs.js.JSON

object Routes {

  // TODO: Move page to TopLevel.
  sealed trait Page
  case object Root extends Page
  case object Login extends Page
  case object Home extends Page
  case object SettingsPage extends Page

  case class LoginMsg(msg: Option[String], returnPage: Option[String]) extends Page {
    def toUrlFrag = {
      Seq(msg.map(m => s"msg=$m"), returnPage.map(m => s"return_url=$m")).flatten.mkString("&")
    }
  }

  case class QuotePage(step: Option[Int], zip: Option[String] = None) extends Page
  case class QuoteA1Page(step: Option[Int]) extends Page
  case class ClaimsPage(id: Option[String]) extends Page

  //==================== Constants
  val baseUrl = BaseUrl.fromWindowOrigin_/ / "app"
  val env     = Env.makeEnvironment
  val spinner = LoggedInSpinner.C()
  val layout = { (c: RouterCtl[Page], r: Resolution[Page]) =>
    LocaleProvider(LocaleProvider.enUS)(r.render()): VdomElement
  }

  val routerConfig = RouterConfigDsl[Page].buildConfig { dsl =>
    import dsl._

    val urlRegex = """([^&=]*)=([^&]*)&?""".r

    def parseLogin(urlFrag: String): Option[LoginMsg] = {
      val r = urlRegex.findAllMatchIn(urlFrag).map {
        case urlRegex(name, key) => Some(name -> key)
        case _                   => None
      }

      val res: Map[String, String] = r.toList.flatten.toMap
      Some(LoginMsg(res.get("msg"), res.get("return_url")))
    }

    val partPath = string("[^/]+")

    (
      emptyRule
        | staticRoute(root, Root) ~> renderR(r => Lander.C(Lander.Props(r)))
        | staticRoute("/home", Home) ~> renderR(
          r => baseViewer[BaseViewerFrag](relay.generated.Home, r, rp => views.Home.Container.rprops(rp)(r)))
        | staticRoute("/login", Login) ~> renderR(p => views.Login.FormC(views.Login.Props(p, env)))
        | staticRoute("/settings", SettingsPage) ~> renderR(
          r => baseViewer[BaseViewerFrag](relay.generated.Settings, r, rp => views.Settings.Container.rprops(rp)(r)))
        | dynamicRouteCT("/login?" ~ remainingPath.pmap(parseLogin)(_.toUrlFrag)) ~> dynRenderR(
          (page, ctl) => views.Login.FormC(views.Login.Props(ctl, env, Some(page))))
        | dynamicRouteCT("/quote" ~ (("/" ~ int).option ~ ("/" ~ string(".....")).option).caseClass[QuotePage]) ~> dynRenderR(
          (page, ctl) => QuoteParent.QR(ctl, env, page))
        | dynamicRouteCT("/quoteme" ~ ("/" ~ int).option.caseClass[QuoteA1Page]) ~> dynRenderR(
          (page, ctl) => QuoteA1View.QR(ctl, env, page))
        | dynamicRouteCT("/claim" ~ ("/" ~ partPath).option.caseClass[ClaimsPage]) ~> dynRenderR(
          (page, ctl) => ClaimView.QR(ctl, env, page))
    ).notFound(redirectToPage(Root)(Redirect.Replace))
      .renderWith(layout)
  }

  @gql("""
    query Home($after: String, $first: Int!) {
      viewer {
        id
        ...Home_viewer
      }
    }
  """)
  object homeQ extends js.Object

  @gql("""
    query Settings($after: String, $first: Int!) {
      viewer {
        id
        ...Settings_viewer
      }
    }
  """)
  object settingsQ extends js.Object

  def baseViewer[P <: js.Object](query: TaggedNode, ctl: RouterCtl[Page], fn: (P) => VdomElement) =
    QueryRenderer
      .builder[P](environment = env, query = query, variables = new js.Object {
        val after: String = null
        val first         = 100
      })
      .renderEither {
        case Some(Left(queryError)) =>
          val err = queryError.errors
          if (err.isDefined) {
            // TODO: Probably move this to an implicit
            val lre = LocalRelayError(err.get)
            if (lre.isAuthError) {

              // TODO: Probably look at a different approach
              ctl.set(LoginMsg(Some("auth"), None)).map(_ => spinner).runNow()
            } else {

              dom.console.error("Server issue", queryError)
              Alert(message = s"Sorry we encountered a problem")
            }
          } else {
            dom.console.error("Unknown issue", queryError)
            <.div("Unknown issue.")
          }

        case Some(Right(props)) =>
          //TODO: Need to fix this, it's kind of absurd
          if (props.asInstanceOf[js.Dynamic].viewer == null) {
            // THIS IS a log out
            ctl.set(Root).map(_ => spinner).runNow()
          } else {
            fn(props)
          }

        case None =>
          spinner
      }
      .build
}
