package web.views

import ant.cps.Layout.{Content, Footer, Header, Sider}
import ant.cps._
import dispalt.relay.gql
import japgolly.scalajs.react._
import japgolly.scalajs.react.extra.router.RouterCtl
import japgolly.scalajs.react.vdom.html_<^._
import org.scalajs.dom
import relay.components._
import relay.generated.Core_viewer
import relay.graphql.TaggedNode
import web.base.all._
import web.base.data.BaseViewerFrag
import web.cps.{PageFooter, TopSearch}
import web.mutation.LogoutMutation
import web.nav.{NavI, NavItem, SubNavItem}
import web.react.RelayBackend
import web.routes.Routes
import web.routes.Routes.Page

import scala.scalajs.js
import scala.scalajs.js.annotation.ScalaJSDefined
import js.JSConverters._

object Core {

  case class State(collapse: Boolean)

  val colors = Seq("#f56a00", "#7265e6", "#ffbf00", "#00a2ae")

  val Nav: Seq[NavI[Page]] = Seq(NavItem("Home", "home", "home", Routes.Home),
                                 NavItem("Create Policy", "create-policy", "file", Routes.QuotePage(Some(1))),
                                 NavItem("File a Claim", "create-claim", "coffee", Routes.ClaimsPage(None)),
                                 SubNavItem("Settings",
                                            "settings",
                                            "setting",
                                            NavItem("Profile", "settings.profile", "user", Routes.SettingsPage) :: Nil))

  case class Props(fluid: Boolean, ctl: RouterCtl[Page], p: Page)

  class Backend(val $ : BackendScope[Props, Unit]) extends RelayBackend[BaseViewerFrag] {

    lazy val navMap: Map[String, NavI[Page]] = NavI.traverse(Nav, Map.empty)

    val onSiderSelect = { (select: Menu.SelectMenuCallback) =>
      navMap.get(select.key).foreach {
        case nav: NavItem[Page]   => $.props.flatMap(_.ctl.set(nav.page)).runNow()
        case sn: SubNavItem[Page] => dom.console.log("Clicked a subnav")
      }
    }

    def logoutConfirm = Callback {
      LogoutMutation.commit(relayProps.viewer.id, relay.environment).foreach {
        case Left(err) => println(err)
        case Right(f)  => dom.console.log(f)
      }
    }

    val onTopSelect = { (select: Menu.SelectMenuCallback) =>
      select.key match {
        case "logout" =>
        case _        => dom.console.log("unbound action")
      }
    }

    def render(p: Props, pc: PropsChildren) = {
      val (defaultOpen, defaultSelected) = navMap.values
        .collect {
          case k: NavItem[Page] if k.page == p.p =>
            val res: js.Array[String] = {
              val split = k.key.split("[.]", 2)

              if (split.size == 2) js.Array(split(0))
              else js.Array()
            }

            (res, k.key)
        }
        .headOption
        .unzip

      val usr = relayProps.viewer.user.toOption
      val name = (for {
        fn <- usr.map(_.user.firstName)
        ln <- usr.flatMap(_.user.lastName.headOption)
      } yield s"$fn $ln").getOrElse("")

      val containerClass = if (p.fluid) coreLo("fluid") else coreLo("fixed")

      // T
      Layout(new Layout.Opts {}.ex(^.minHeight := "100vh"))(
        Sider(new Sider.Opts {
          override val breakpoint     = "lg"
          override val collapsedWidth = 0
        })(<.div(^.className := "logo")(<.img(^.src := "/assets/images/logo-mark-color-32x.png")),
           Menu(theme = "dark",
                mode = "inline",
                defaultSelectedKeys = defaultSelected.toJSArray,
                defaultOpenKeys = defaultOpen.flatten.toJSArray,
                onSelect = onSiderSelect)(Nav.toVdomArray(NavI.createNav))),
        Layout(
          Header((new Header.Opts {})(^.background := "#fff", ^.padding := "0"))(
            <.div(^.id := "search-box")(TopSearch.C()),
            <.div(^.id := "profile")(Avatar.C(new Avatar.Opts {
              override val size = "large"
            }.ex(^.backgroundColor := colors(math.abs(name.##) % colors.length)))(name)),
            <.div(^.className := coreLo("header"))(Menu(mode = "horizontal", id = "nav", onSelect = onTopSelect)(
              Menu.Item(key = "logout")(PopConfirm(new PopConfirm.Opt {
                override val title     = js.defined("Confirm you'd like to logout.")
                override val placement = "bottomRight"
                override val onConfirm = js.defined(logoutConfirm.toJsFn)
              })(<.div(Icon("logout"), "Logout")))))),
          Content(new Content.Opts {})(<.div(^.className := containerClass)(pc)),
          PageFooter()))
    }
  }

  @gql("""
      fragment Core_viewer on Viewer {
        id
        user {
          user {
            id
            firstName
            lastName
         }
         roles
         permissions
        }
      }
    """)
  object localViewer extends TaggedNode

  //
  val C = ScalaComponent
    .builder[Props]("Core")
    .stateless
    .renderBackendWithChildren[Backend]
    .buildRelay[BaseViewerFrag]

  val Container = FragmentContainer.buildRelay(C, "viewer" -> Core_viewer.query)

  override def toString = "[Core]"
}
