package web.admin.cps

import ant.cps.{Card, Table}
import ant.cps.Table.ColumnProps
import dispalt.relay.gql
import japgolly.scalajs.react._
import japgolly.scalajs.react.extra.router.RouterCtl
import japgolly.scalajs.react.vdom.html_<^._
import moment.Moment
import web.react.{RelayBackend, RelayBackendRefetch}
import relay.components._
import web.admin.data.BaseViewerFrag
import web.admin.routes.{Page, UserPage}
import web.base.data.{PolicyFrag, SangriaEdge}
import web.cps.Sectional

import scala.scalajs.js
import scala.scalajs.js.JSON

object PolicyAdminList {
  type RT = BaseViewerFrag

  val C = ScalaComponent
    .builder[RouterCtl[Page]]("PolicyAdminList")
    .stateless
    .renderBackend[Backend]
    .buildRelay[RT]

  @gql("""
      fragment PolicyAdminList_viewer on Viewer {
        policies(after: $after, first: $first) @connection(key: "PolicyAdminList_policies") {
          edges {
            node {
              quoteId policyId userId userIdC
              email
              activatedMs
              contractValue
              details {
                details {
                  namedInsured { firstName lastName }
                  insuredLocation { lat lng formattedAddress }
                }
                sec1 { coverageC coverageD deductible }
                sec2 { coverageE coverageF }
              }
            }
            cursor
          }
          pageInfo {
            hasNextPage
            endCursor
          }
        }
      }
       """)
  object pal_viewer extends TaggedNode

  @gql("""
      query PolicyAdminList($after: String, $first: Int) {
        viewer {
          ...PolicyAdminList_viewer
        }
      }
    """)
  object pal_query extends TaggedNode

  val Container = RefetchContainer.buildRelay(C, pal_query, "viewer" -> pal_viewer)

  class Backend(val $ : BackendScope[RouterCtl[Page], _]) extends RelayBackendRefetch[RT] {

    def render(ctl: RouterCtl[Page]) = {
      Sectional(new Card.Opts {
        override val title = "Policy List"
      })(Table(new Table.Opts[SangriaEdge[PolicyFrag]] {

        override val expandedRowRender = js.defined { (r: SangriaEdge[PolicyFrag]) =>
          <.pre(JSON.stringify(r.node.details, space = 2)).rawNode
        }
        override val size = "small"
        override val rowKey = { r: SangriaEdge[PolicyFrag] =>
          r.cursor
        }: js.Function1[SangriaEdge[PolicyFrag], String]

        val dataSource = relayProps.viewer.policies.edges
        val columns = js.Array[ColumnProps[SangriaEdge[PolicyFrag]]](ColumnProps("Id", "id", render = js.defined {
          (_, rec, _) =>
            <.code(rec.node.policyId).rawNode
        }), ColumnProps("Activated", "activated", render = js.defined { (_, rec, _) =>
          Moment(rec.node.activatedMs).fromNow()
        }), ColumnProps("Price / Yr", render = js.defined { (_, rec, _) =>
          f"$$${rec.node.contractValue}%.2f USD"
        }), ColumnProps("Details (c,d)", render = js.defined { (_, rec, _) =>
          <.span(List(rec.node.details.sec1.coverageC.toString, rec.node.details.sec1.coverageD.toString)
            .mkString(",")).rawNode
        }), ColumnProps("Owner", "owner", render = js.defined { (_, rec, _) =>
          <.code(ctl.link(UserPage(rec.node.userIdC))(rec.node.userId)).rawNode
        }))
      }))
    }
  }
}
