package web.admin.cps

import ant.cps._
import ant.cps.Table.ColumnProps
import dispalt.relay.gql
import japgolly.scalajs.react._
import japgolly.scalajs.react.extra.router.RouterCtl
import japgolly.scalajs.react.vdom.VdomElement
import japgolly.scalajs.react.vdom.html_<^._
import moment.Moment
import org.scalajs.dom
import relay.components._
import web.admin.data._
import web.base.all._
import web.admin.routes.{Page, UserPage}
import web.base.data.SangriaEdge
import web.react.{RelayBackend, RelayBackendRefetch}

import scala.scalajs.js
import scala.scalajs.js.UndefOr

object UserList {

  class Backend(val $ : BackendScope[RouterCtl[Page], _]) extends RelayBackendRefetch[BaseViewerFrag] {

    val searchClick = (e: String) =>
      Callback {
        val st =
          if (e == null || e.isEmpty) null
          else e
        relay.refetch({ f: js.Dictionary[js.Any] =>
          f.update("searchTerm", st)
          f
        }: js.Function1[js.Dictionary[js.Any], js.Dictionary[js.Any]])
        ()
    }

    def resetClick(e: ReactEvent): Callback = e.preventDefaultCB >> Callback {
      relay.refetch(js.Dictionary[js.Any]("searchTerm" -> null, "first" -> 10, "after" -> null))
    }

    // Foo
    def render(ctl: RouterCtl[Page]) = {
      Card(new Card.Opts {
        override val title     = "Users"
        override val className = "card-thck"
      })(
        Row(Row.Opt, ^.marginBottom := "10px")(Input.Search(new Input.Search.Opt {
          override val onSearch = searchClick.toJsCallback
        })),
        Table(new Table.Opts[SangriaEdge[FullUserFrag]] {
          override val size = "small"
          override val rowKey = { r: SangriaEdge[FullUserFrag] =>
            r.node.id
          }: js.Function1[SangriaEdge[FullUserFrag], String]
          val columns = js.Array[ColumnProps[SangriaEdge[FullUserFrag]]](ColumnProps(title = "Id", render = js.defined {
            (_, rec, _) =>
              <.code(ctl.link(UserPage(rec.node.id))(rec.node.rawId)).rawNode
          }), ColumnProps(title = "Name", render = js.defined { (f, rec, n) =>
            rec.node.user.firstName + " " + rec.node.user.lastName
          }), ColumnProps(title = "Email", render = js.defined { (f, rec, n) =>
            rec.node.user.email
          }), ColumnProps(title = "Policies", render = js.defined { (_, rec, _) =>
            rec.node.policies.edges.toVdomArray(n => <.code(^.key := n.node.policyId)(n.node.policyId)).rawNode
          }), ColumnProps(title = "Created", render = js.defined { (_, rec, _) =>
            Moment(rec.node.user.timeCreatedMs).fromNow()
          }))
          val dataSource = relayProps.viewer.users.edges
        }))
    }
  }

  //
  @gql("""
    fragment UserList_viewer on Viewer {
      id
      users(after: $after, first: $first, searchTerm: $searchTerm) @connection(key: "UserList_users")  {
        edges {
          node {
            id
            rawId
            permissions
            roles
            user {
              firstName
              lastName
              email
              timeCreatedMs
              timeUpdatedMs
              permissionsVersion
            }
            policies(after: $after, first: $first) @connection(key: "UserList_policies") {
              edges {
                node {
                  policyId
                  activatedMs
                }
              }
            }
          }
          cursor
        }
        pageInfo { hasNextPage endCursor }
      }
    }
  """)
  object ppc_viewer extends TaggedNode

  @gql("""
      query UserListRefetchQuery($searchTerm: String, $after: String, $first: Int) {
        viewer {
          ...UserList_viewer
        }
      }
    """)
  object ul_queryrc extends TaggedNode

  val C = ScalaComponent
    .builder[RouterCtl[Page]]("UserList")
    .stateless
    .renderBackend[Backend]
    .buildRelay[BaseViewerFrag]

  val Container = RefetchContainer.buildRelay(C, ul_queryrc, "viewer" -> ppc_viewer)
}
