package web.admin.routes

import ant.cps.{Alert, LocaleProvider}
import dispalt.relay.gql
import japgolly.scalajs.react._
import relay.components._
import japgolly.scalajs.react.extra.router._
import japgolly.scalajs.react.component.Generic._
import japgolly.scalajs.react.vdom.html_<^._
import org.scalajs.dom
import relay.QueryRenderer
import relay.graphql.TaggedNode
import web.admin.data.BaseViewerFrag
import web.admin.run.Env
import web.admin.views.SpecificUserView.BaseFullUserFrag
import web.admin.views.{HomeView, Login, SpecificUserView}
import web.base.all._
import web.base.data.LocalRelayError
import web.cps.{LoggedInSpinner, PageSpinner}

import scala.scalajs.js.JSConverters._
import scala.scalajs.js
import scala.scalajs.js.Dynamic.literal
import scala.scalajs.js.{JSON, JavaScriptException}
import scala.scalajs.runtime.UndefinedBehaviorError

sealed trait Page
case object Home               extends Page
case object SettingsPage       extends Page
case class UserPage(s: String) extends Page

case class LoginMsg(msg: Option[String], returnPage: Option[String]) extends Page {
  def toUrlFrag = {
    Seq(msg.map(m => s"msg=$m"), returnPage.map(m => s"return_url=$m")).flatten.mkString("&")
  }
}

case class QuotePage(step: Option[Int]) extends Page

object Routes {

  //==================== Constants
  val baseUrl = BaseUrl.fromWindowOrigin_/ / "admin"
  val env     = Env.makeEnvironment
  val spinner = LoggedInSpinner.C()
  val layout = { (c: RouterCtl[Page], r: Resolution[Page]) =>
    LocaleProvider(LocaleProvider.enUS)(
      r.render()
    ): VdomElement
  }

  val routerConfig = RouterConfigDsl[Page].buildConfig { dsl =>
    import dsl._

    val urlRegex = """[?]?([^&=]*)=([^&]*)&?""".r

    def parseLogin(urlFrag: String): Option[LoginMsg] = {
      val r = urlRegex.findAllMatchIn(urlFrag).map {
        case urlRegex(name, key) => Some(name -> key)
        case _                   => None
      }

      val res: Map[String, String] = r.toList.flatten.toMap
      Some(LoginMsg(res.get("msg"), res.get("return_url")))
    }

    (
      emptyRule
        | staticRoute(root, Home) ~> renderR(
          r => baseViewer[BaseViewerFrag](homeQ, r, rp => HomeView.Container.rprops(rp)(HomeView.Props(r)))
        )
        | dynamicRouteCT("/user" / remainingPath.caseClass[UserPage]) ~> dynRenderR[UserPage, VdomElement] {
          (page, r) =>
            baseViewer[BaseFullUserFrag](
              userViewQ,
              r, { rp =>
                val rsp = literal(
                  fullUser = rp.node
                )
                // Foo
                SpecificUserView.Container.rprops(rp.mergeLeftType(rsp))(r)
              },
              Some(new js.Object {
                val id          = page.s
                val policyFirst = 100
                val partyFirst  = 100
              })
            )
        }
        | dynamicRouteCT("/login" ~ remainingPathOrBlank.pmap(parseLogin)(_.toUrlFrag)) ~> dynRenderR(
          (page, ctl) => Login.FormC(Login.Props(ctl, env, page))
        )
    ).notFound(redirectToPage(Home)(Redirect.Replace))
      .renderWith(layout)
  }

  @gql("""
    query Home($after: String, $first: Int!, $searchTerm: String) {
      viewer {
        id
        ...HomeView_viewer
      }
    }
  """)
  object homeQ extends js.Object

  // Test
  @gql("""
    query UserView($id: ID!, $partyFirst: Int!, $policyFirst: Int!) {
      viewer {
        ...SpecificUserView_viewer
        id
      }
      node(id: $id) {
        ... on FullUser {
          ...SpecificUserView_fullUser
        }
      }
    }
  """)
  object userViewQ extends js.Object
  //

  def baseViewer[P <: js.Object](query: TaggedNode,
                                 ctl: RouterCtl[Page],
                                 fn: (P) => VdomElement,
                                 variables: Option[js.Object] = None) =
    QueryRenderer
      .builder[P](
        environment = env,
        query = query,
        variables = variables.getOrElse[js.Object](new js.Object {
          val after: String = null
          val first         = 100
        })
      )
      .renderEither {
        case Some(Left(queryError)) =>
          val err = queryError.errors
          if (err.isDefined) {
            // TODO: Probably move this to an implicit
            val lre = LocalRelayError(err.get)
            if (lre.isAuthError) {

              // TODO: Probably look at a different approach
              ctl.set(LoginMsg(Some("auth"), None)).map(_ => spinner).runNow()
            } else {

              dom.console.error("Server issue", queryError)
              Alert(message = s"Sorry we encountered a problem")
            }
          } else {
            dom.console.error("Unknown issue", queryError)
            <.div("Unknown issue.")
          }

        case Some(Right(props)) =>
          //TODO: Need to fix this, it's kind of absurd
          if (props.asInstanceOf[js.Dynamic].viewer == null) {
            // THIS IS a log out
            ctl.set(LoginMsg(Some("logout"), None)).map(_ => spinner).runNow()
          } else {
            fn(props)
          }

        case None =>
          spinner
      }
      .build
}
