package web.admin.views

import dispalt.relay.gql
import japgolly.scalajs.react._
import japgolly.scalajs.react.extra.router.RouterCtl
import monix.execution.Ack
import monix.execution.cancelables.SingleAssignmentCancelable
import monix.reactive.OverflowStrategy
import org.scalajs.dom
import relay.components._
import relay.runtime.RecordSourceSelectorProxy
import relay.runtime.RelayRuntime.{ConnectionHandler, ViewerHandler}
import relay.runtime.subscription.{RequestRelaySubscription, RequestRelayVars}
import vdom.html_<^._
import web.admin.cps._
import web.admin.data.{BaseViewerFrag, FullUserFrag}
import web.admin.routes.Page
import web.cps.Sectional
import web.react.RelayBackend
import web.base.all._

import scala.scalajs.js

object HomeView {

  case class Props(ctl: RouterCtl[Page])
  case class State()

  class Backend(val $ : BackendScope[Props, State]) extends RelayBackend[BaseViewerFrag] {
    val querySeed  = now
    val cancelable = SingleAssignmentCancelable()

    def willMount =
      Callback {
        cancelable := RequestRelaySubscription[js.Object](relay.environment,
                                                          sub,
                                                          new RequestRelayVars {
                                                            val seed = querySeed.toString
                                                          }, { rsp: RecordSourceSelectorProxy =>
                                                            val viewer        = rsp.get(relayProps.viewer.id).get
                                                            val operatorsLive = viewer.getLinkedRecord("operatorsLive")
                                                            val connection = ConnectionHandler
                                                              .getConnection(viewer, "Pending_approvals")
                                                              .get

                                                            rsp
                                                              .getRootField("operatorEvents")
                                                              .foreach {
                                                                root =>
                                                                  // Do the operator connect/disconnect
                                                                  root.getLinkedRecord("users").foreach { f =>
                                                                    if (f != null)
                                                                      operatorsLive.foreach(viewer =>
                                                                        viewer.copyFieldsFrom(f))
                                                                  }
                                                                  // Do the removal/add
                                                                  root.getLinkedRecord("node").foreach { f =>
                                                                    if (f != null) {
                                                                      f.getValueStr("__typename") match {
                                                                        case "OperatorReviewed" =>
                                                                          ConnectionHandler.deleteNode(
                                                                            connection,
                                                                            f.getValueStr("reviewId"))
                                                                        case "OperatorReviewPending" =>
                                                                          val edge = ConnectionHandler.createEdge(
                                                                            rsp,
                                                                            connection,
                                                                            f,
                                                                            "OperatorReviewPendingEdge")
                                                                          ConnectionHandler.insertEdgeBefore(connection,
                                                                                                             edge)
                                                                      }
                                                                      dom.console.log(f.getValueStr("__typename"))
                                                                    }
                                                                  }
                                                              }
                                                            ()
                                                          })(OverflowStrategy.DropOld(100)).subscribe(_ => Ack.Continue)
      } >> Callback.log("Mount")

    def unmount =
      Callback {
        cancelable.cancel()
      } >> Callback.log("Unmount")

    def render(p: Props, s: State) = {
      //

      AdminCore.Container.rprops(relayProps)(AdminCore.Props(fluid = true, ctl = p.ctl, p = web.admin.routes.Home))(
        Sectional(ConnectionStats.C(ConnectionStats.Props(relayProps.viewer.operatorsLive.edges.map(_.node).toList))),
        <.div(^.marginBottom := "20px")(OperatorReviewList.Container.rprops(relayProps)()),
        <.div(^.marginBottom := "20px")(PolicyAdminList.Container.rprops(relayProps)(p.ctl)),
        <.div(^.marginBottom := "20px")(UserList.Container.rprops(relayProps)(p.ctl)))
    }
  }

  @gql("""
      subscription HomeView_sub($seed: String!) {
        operatorEvents(seed: $seed) {
          users {
            edges { node {
              id
              user {
                email
                firstName
              }
            } }
          }
          node {
            __typename
            ... on OperatorReviewPending {
              id rawId userId
              fullUser {
                id
                user {
                  firstName
                }
              }
              when
              pc {
                id
                before
                after
              }
            }
            ... on OperatorReviewed {
              id rawId details approved reviewId

            }
          }
        }
      }
    """)
  object sub extends TaggedNode

  @gql("""
      fragment HomeView_viewer on Viewer {
        id
        user {
          user {
            id
            firstName
            lastName
          }
          roles
          permissions
        }
        operatorsLive { edges { node { id user { firstName email } } } }
        ...AdminCore_viewer
        ...OperatorReviewList_viewer
        ...UserList_viewer
        ...PolicyAdminList_viewer
      }
    """)
  object localViewer extends TaggedNode

  val C = ScalaComponent
    .builder[Props]("HomeView")
    .initialState(State())
    .renderBackend[Backend]
    .componentDidMount(_.backend.willMount)
    .componentWillUnmount(_.backend.unmount)
    .buildRelay[BaseViewerFrag]

  val Container = FragmentContainer.buildRelay(C, "viewer" -> localViewer)
}
