package web.admin.views

import ant.cps.Form.{FormBackend, FormComponent}
import ant.cps._
import japgolly.scalajs.react._
import japgolly.scalajs.react.extra.router.RouterCtl
import japgolly.scalajs.react.vdom.html_<^._
import monix.eval.Task
import relay.runtime.store.RelayEnvironment
import shared.models.ErrorKeys
import web.admin.mutation.LoginMutation
import web.admin.routes.{Home, LoginMsg, Page}
import web.base.all._
import web.base.data.LocalRelayError
import web.views.Clean

import scala.concurrent.Future
import scala.scalajs.js

object Login {

  /* Relay Environment, Probably won't need this when we start displaying if they are logged
   * in here. */
  case class Props(ctl: RouterCtl[Page], env: RelayEnvironment, msg: LoginMsg)

  case class State(err: LocalRelayError)

  class Backend(val $ : BackendScope[Props, Unit]) extends FormBackend {

    val onSelect = { f: Menu.SelectMenuCallback =>
      $.props.flatMap(_.ctl.set(Home)).logResult.runNow()
    }

    val formSubmit = { r: ReactEvent =>
      r.preventDefault()
      $.props.toTask.flatMap { p =>
        for {
          vf <- validateFields[js.Object]()
          _ <- vf match {
            case Left(_) => Task.unit
            case Right(res) =>
              LoginMutation.commit(p.env, res) flatMap {
                case Left(f) =>
                  Task(form.setFields(f.jsErrors))
                case Right(_) =>
                  p.ctl.set(Home).toTask
              }
          }
        } yield ()
      }.runAsyncForget
    }

    val hdr = Menu(mode = "horizontal", id = "nav", onSelect = onSelect)(Menu.Item(key = "home")("Home"))

    private def globalFailBanner: VdomNode = {
      val err = form.getFieldError(ErrorKeys.global).asInstanceOf[js.UndefOr[js.Array[String]]]
      err.fold(VdomNode.cast(null))(err =>
        Alert(message = "Error", description = err.join(", "), `type` = "error", showIcon = true))

    }

    private def serverFailureBanner: VdomNode = {
      if (form.getFieldError(ErrorKeys.serverFailure).isDefined)
        Alert(
          message     = "Server Error",
          description = "Something went wrong, please try again shortly.  Someone has been notified.",
          `type`      = "error",
          showIcon    = true)
      else VdomNode.cast(null)
    }

    def render(p: Props) = {

      Clean.C(new Clean.Props(Some(hdr), true))(
        globalFailBanner,
        serverFailureBanner,
        Form((new Form.Opt {
          override val onSubmit = js.defined(formSubmit)
        })(^.maxWidth := "300px", ^.className := "login-form"))(
          Form.Item(form.getFieldDecorator("login", new FormComponent.Opt {
            override val rules = js.defined(js.Array(new FormComponent.RuleOpt {
              override val message = "Please input your email"
              val required         = true
            }))
          })(Input(new Input.Opt {
            override val addonBefore = js.defined(Icon(`type` = "user").raw)
            override val placeholder = "Email"
          }))),
          Form.Item(form.getFieldDecorator("password", new FormComponent.Opt {
            override val rules = js.defined(js.Array(new FormComponent.RuleOpt {
              override val message = "Please input your password"
              val required         = true
            }))
          })(Input(new Input.Opt {
            override val `type`      = "password"
            override val addonBefore = js.defined(Icon(`type` = "lock").raw)
            override val placeholder = "Password"
          }))),
          Form.Item(
            form.getFieldDecorator("remember", new FormComponent.Opt {
              override val valuePropName = "checked"
              override val initialValue  = true
            })(Checkbox("Remember Me")),
            Button(`type` = "primary", htmlType = "submit", className = "login-form-button")("Log in"))))

    }
  }

  val C = ScalaComponent.builder[Props]("Login").stateless.renderBackend[Backend].build

  val FormC = Form.create(C)

}
