package web.admin.views

import ant.cps._
import dispalt.relay.gql
import japgolly.scalajs.react._
import japgolly.scalajs.react.extra.router.RouterCtl
import japgolly.scalajs.react.vdom.html_<^._
import moment.Moment
import org.scalajs.dom
import web.admin.data.{BaseViewerFrag, FullUserFrag, ViewerFrag}
import relay.components._
import web.base.all._
import web.admin.cps.AdminCore
import web.admin.routes.{Home, Page}
import web.cps.{PolicyList, Sectional}

import scala.scalajs.js
import scala.scalajs.js.JSON
import scala.scalajs.js.annotation.ScalaJSDefined

object SpecificUserView {

  @js.native
  trait BaseFullUserFrag extends BaseViewerFrag {
    val fullUser: js.UndefOr[FullUserFrag]
  }

  @gql("""
      fragment SpecificUserView_fullUser on FullUser {
        rawId
        id
        user { rawId id firstName lastName email timeCreatedMs }
        roles
        permissions
        policies(first: $policyFirst) @connection(key: "SpecificUserView_policies") {
          edges {
            node {
              contractValue
              policyId activatedMs
              details {
                details {
                  namedInsured { firstName lastName }
                  insuredLocation { lat lng formattedAddress }
                }
                sec1 { coverageC coverageD deductible }
                sec2 { coverageE coverageF }
              }
            }
          }
        }
        partyEvents(first: $partyFirst) @connection(key: "SpecificUserView_partyEvents") {
          edges {
            node { id userId when tc }
          }
          pageInfo { hasNextPage }
        }
      }
    """)
  object fullUser extends TaggedNode

  @gql("""
      fragment SpecificUserView_viewer on Viewer {
        id
        user {
          id
          user { firstName lastName }
        }
        ...AdminCore_viewer
      }
  """)
  object viewer extends TaggedNode

  def row(name: String, value: VdomNode) = {
    Row()(Col(new Col.Opt {
      override val span = 12
    })(name), Col(new Col.Opt {
      override val span = 12
    })(value))
  }

  val C = ScalaComponent
    .builder[RouterCtl[Page]]("SpecificUserView")
    .stateless
    .render { r =>
      val rb = r.relayBackend[BaseFullUserFrag]
      val fu = rb.relayProps.fullUser.get
      AdminCore.Container.rprops(rb.relayProps)(AdminCore.Props(fluid = true, ctl = r.props))(
        Sectional.C(Sectional.Props(Card.Opts))(
          <.h2(s"${fu.user.firstName} ${fu.user.lastName} - ${fu.roles.mkString(", ")}", Icon("refresh")),
          row("Id", fu.rawId),
          row("Lookup Id", fu.id),
          row("First Name", fu.user.firstName),
          row("Last Name", fu.user.lastName),
          row("Email", fu.user.email),
          row("Created", Moment(fu.user.timeCreatedMs).fromNow()),
          row("Permissions", fu.permissions.mkString(", ")),
          row("Num of Policies", fu.policies.edges.length),
          row("Num of Events", fu.partyEvents.edges.length)),
        PolicyList(fu.policies.edges.map(_.node)),
        Sectional(Timeline {
          var i = 0
          fu.partyEvents.edges.toVdomArray { edge =>
            val msg = edge.node.tc
            i += 1
            Timeline.Item.C.withKey(i)(Timeline.Item.Opts)(Moment(edge.node.when).fromNow(), " - ", msg)
          }
        }))
    }
    .buildRelay[BaseFullUserFrag]

  val Container = FragmentContainer.buildRelay(C, "fullUser" -> fullUser, "viewer" -> viewer)
}
